/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.autolayout;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * 
 * @author luoguo
 */
public class PointUtil {

	/**
	 * 返回开始模块
	 * 
	 * @return
	 * @throws Exception
	 */
	public static Node getStartNode(ArrayList<Node> nodeList) throws Exception {
		for (Node node : nodeList) {
			if (node.getType() == Node.TYPE_START) {
				return node;
			}
		}
		throw new Exception("This is no START node");
	}

	/**
	 * 返回结束模块
	 * 
	 * @return
	 * @throws Exception
	 */
	public static Node getEndNode(ArrayList<Node> nodeList) throws Exception {
		for (Node node : nodeList) {
			if (node.getType() == Node.TYPE_END) {
				return node;
			}
		}
		throw new Exception("This is no END node");
	}

	/**
	 * 得到垂直向上方向离节点最近的图元
	 * 
	 * @param m
	 * @return
	 */
	public static Node getNearestUpNode(Node m) {
		Node ret = null;
		for (Link link : m.linkList) {
			if (ret == null || link.getAnotherNode(m).point.y > ret.point.y) {
				ret = link.getAnotherNode(m);
			}
		}
		return ret;
	}

	/**
	 * 得到垂直向下方向离节点最近的图元
	 * 
	 * @param m
	 * @return
	 */
	public static Node getNearestDownNode(Node m) {
		Node ret = null;
		for (Link link : m.linkList) {
			if (ret == null || link.getAnotherNode(m).point.y < ret.point.y) {
				ret = link.getAnotherNode(m);
			}
		}
		return ret;
	}

	/**
	 * 交换两个图元坐标点位置
	 * 
	 * @param m
	 * @param n
	 */
	public static void exchangePosition(Node m, Node n) {
		Point p = m.point;
		m.point = n.point;
		n.point = p;
	}

	/**
	 * 下移
	 * 
	 * @param m
	 */
	public static void moveDown(Node m, ArrayList<Node> nodeList) {
		if (isHasNodeInPosition(m.point.x, m.point.y + 1, nodeList)) {
			Node t = getNodeInPosition(m.point.x, m.point.y + 1, nodeList);
			if (t != null) {
				moveDown(t, nodeList);
			}
		}
		m.point.setPosition(m.point.x, m.point.y + 1);
	}

	/**
	 * 左移
	 * 
	 * @param m
	 */
	public static void moveLeft(Node m, ArrayList<Node> nodeList) {
		if (isHasNodeInPosition(m.point.x - 1, m.point.y, nodeList)) {
			Node t = getNodeInPosition(m.point.x - 1, m.point.y, nodeList);
			if (t != null) {
				moveDown(t, nodeList);
			}
		}
		m.point.setPosition(m.point.x - 1, m.point.y);
	}

	/**
	 * 右移
	 * 
	 * @param m
	 */
	public static void moveRight(Node m, ArrayList<Node> nodeList) {
		if (isHasNodeInPosition(m.point.x + 1, m.point.y, nodeList)) {
			Node t = getNodeInPosition(m.point.x + 1, m.point.y, nodeList);
			if (t != null) {
				moveDown(t, nodeList);
			}
		}
		m.point.setPosition(m.point.x + 1, m.point.y);
	}

	/**
	 * 验证是否存在指定坐标的图元
	 * 
	 * @param x
	 * @param y
	 * @return
	 */
	public static boolean isHasNodeInPosition(int x, int y,
			ArrayList<Node> nodeList) {
		for (Node m : nodeList) {
			if (m.point.isSamePosition(x, y)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * 获得指定坐标点的图元
	 * 
	 * @param x
	 * @param y
	 * @return
	 */
	public static Node getNodeInPosition(int x, int y, ArrayList<Node> nodeList) {
		for (Node m : nodeList) {
			if (m.point.isSamePosition(x, y)) {
				return m;
			}
		}
		return null;
	}

	/**
	 * 判断是否还包含不在start->end线路没有定位的点
	 * 
	 * @param noMainPathPointNodeList
	 * @return
	 */
	public static boolean isConstainNoArangeNode(
			ArrayList<Node> noMainPathPointNodeList) {
		for (Node m : noMainPathPointNodeList) {
			if (m.point.y == 0)
				return true;
		}
		return false;
	}

	/**
	 * 查找两条主线间的相同模块
	 * 
	 * @return
	 */
	public static ArrayList<Node> findSameNode(ArrayList<Node> list1,
			ArrayList<Node> list2) {
		ArrayList<Node> result = new ArrayList<Node>();
		HashMap<Node, Node> map = new HashMap<Node, Node>();
		for (Node m : list1) {
			if (m.getType() == Node.TYPE_COMMON) {
				map.put(m, m);
			}
		}
		for (Node m : list2) {
			if (m.getType() == Node.TYPE_COMMON && map.get(m) != null) {
				result.add(m);
			}
		}
		return result;
	}

}
